/*
****************************************************************************
* R-T-V meter v2,0                                                         *
* Mereni otaček - teploty - napeti pro motocykl - zobrazeni na 2x LCD      *
* Datum: 02_2018                       uP: ATmega168, Arduino mini pro     *
* Krystal:  16MHz               prekladač: Arduino v1.6.5                  *
****************************************************************************
Program realizující měření otáček spalovacího motoru, teplotu motoru,
napětí akumulátoru a měření venkovní teploty.

Měření otáček s rozsahem 0 až 50000ot/min. Pevně nastavena časová základna s časem 1s,
během níž se načítají impulsy na pinu D2 (INT0), na kterém je aktivní externí přerušení. 
Bude-li měřená a vypočtená hodnota otáček vyšší než 50tis. ot/min, zobrazí se symboly
"^^^^^" udávající, že měřená hodnota je vyšší než nastavený rozsah.
Pomocí propojky Volba2 (D7) je možné zobrazit měřené otáčky se zaokrouhlením po 50ti ot/min.
Je-li propojka Volba2 - spojená (log.0), je aktivní zaokrouhlení po 50ot/min.
Výhodou zobrazení se zaokrouhlením je, že se rychleji odečítá aktuální otáčky, protože
číslo se mění pomaleji právě z důvodu hystereze +/- 50ot/min.
Nevýhodou méně přesné zobrazení měřených otáček. 
Impulzní signál otáček je zaveden na digitální vstup D2 (INT0) u kterého je nastaveno
vnější prerušení. Měří se čas, během něhož se načítají impulzy. Je-li měřený čas 1 sekunda,
zastaví se přerušovací systém a vyvolá se výpočet otáček dle naměřených impulzů, počtu
pulzů na otáčku atd. 
Pomocí dvou propojek (Volba3-D8 a Volba4-D9) je možné nastavit počet pulzů na otáčku motoru na:
Volba3 - rozpojená (log.1), Volba4 - rozpojená (log.1) = 1 puls na otačku
Volba3 - spojená (log.0), Volba4 - rozpojená (log.1) = 2 pulsy na otáčku
Volba3 - rozpojená (log.1), Volba4 - spojená (log.0) = 3 pulsy na otáčku
Volba3 - spojená (log.0), Volba4 (log.0) - spojená = 4 pulsy na otáčku
Změna nastavení se projeví po restartu zařízení.

Měření teploty je realizováno pomocí digitálního teploměru DS18(s)20 firmy Dallas.
Teploměr má základní rozlišení po 0,5stC v rozsahu -55 až +125stC.
Rovněz je možné použít přesnější teploměr DS18B20, který má stejný rozsah,
ale disponuje rozlišením po 0,1stC.
Teploměry DS18x20 připojeny na pinech D3 (motor) a D4 (venkovní).

Měření napětí je realizováno pomocí zabudovaného 10ti bitoveho AD prevodniku.
Rozsah měření napětí je 0 až 50V s rozlišením na jedno desetinné místo.
Přizpůsobení rozsahu 0 až 50V na rozsah AD převodníku je realizován pomocí
odporového děliče s poměrem 1:9. Výstup z odporového děliče je zapojen na pin A6. 

Zobrazeni aktuálních hodnot je pomocí dvou LCD displejů 8x2 znaků zapojených následovně:
LCD1 -> RS - A2, E - A1, DB4 - D10, DB5 - D11, DB6 - D12, DB7 - D13
LCD1 -> RS - A2, E - A0, DB4 - D10, DB5 - D11, DB6 - D12, DB7 - D13

Po resetu procesoru jsou na displejích vypsány nápisy "R-T-V METER", verze programu, 
nastavený počet pulzů na otáčku a "pp78c2018". Zobrazí se na 1 sekundu.
Potom již na prvním displeji se zobrazí aktuální otáčky a teplota motoru.
Na druhém displeji se zobrazí aktuální napětí akumulátoru a venkovní teplota.

Volba1 - pin D6, není zatím využit k žádnému nastavení
*/

//Vložení knihoven
#include <LiquidCrystal.h>      //Vložení knihovny pro řízení LCD displeje
#include "OneWire.h"            //Vložení knihovny pro komunikaci typu 1-wire. Umístěná v kořenovém adresáři

//Definice vstupně výstupních pinů
//LiquidCrystal lcd1(12, 13, A2, A3, A4, A5);      //prirazeni pinu (RS, E, D4, D5, D6, D7) z LCD č.1 k arduinu 
//LiquidCrystal lcd2(12, 11, A2, A3, A4, A5);      //prirazeni pinu (RS, E, D4, D5, D6, D7) z LCD č.2 k arduinu
LiquidCrystal lcd1(A2, A1, 10, 11, 12, 13);      //Přiřazení pinů (RS, E, D4, D5, D6, D7) pro LCD č.1 
LiquidCrystal lcd2(A2, A0, 10, 11, 12, 13);      //Přiřazení pinů (RS, E, D4, D5, D6, D7) pro LCD č.2
byte  Voltmetr = A6;                             //A6-Měření napětí - palubní napětí motocyklu
byte  RPM_Pulzy = 2;                             //D2-Vstup čítající pulzy pro měření otáček motoru 
byte  pinINT0 = 0;                               //INT0-Vnější přerušení = digitální pin D2
byte  DS18B20m_Pin = 3;                          //D3-Teploměr DS18x20 - měření teploty motoru, potřeba 4K7 resistor mezi Vcc a D3
OneWire ds1(DS18B20m_Pin);                       //Inicializace komunikace ONEWIRE pro DS18x20 
byte  DS18B20v_Pin = 4;                          //D4-Teploměr DS18x20 - měření venkovní teploty, potřeba 4K7 resistor mezi Vcc a D4
OneWire ds2(DS18B20v_Pin);                       //Inicializace komunikace ONEWIRE pro DS18x20 
byte  Volba1 = 6;                                //D6-Vstupní pin pro volbu nastavení - zatím nevyužito
byte  Volba2 = 7;                                //D7-Vstupní pin pro volbu nastavení zaokrouhlení otáček po 50ot/min
byte  Volba3 = 8;                                //D8-Vstupní pin pro volbu nastavení počtu pulzů na otáčku 1 až 4
byte  Volba4 = 9;                                //D9-Vstupní pin pro volbu nastavení počtu pulzů na otáčku 1 až 4

//Definice dočasných proměnných
float           RPM_VAL = 0;                     //Hodnota naměřených otáček
int             RPM_VAL_INT = 0;                 //Hodnota otáček převedených z Float do Integer pro zaokrouhlení
int             RPM_Modulo = 0;                  //Hodnota otáček po dělení 100 jako zbytek modulo - zaokrouhlení
int             RPM_Pricti = 0;                  //Hodnota o kterou se přicítá po zaokrouhlení otáček
unsigned long   Pocet_Pulzu = 0;                 //Hodnota naměřených počtů impulzů max. 2na32, unsigned int je 2na16
byte            Pulz_Otacka = 1;                 //Hodnota udávající počet pulzů na jednu otáčku
float           Volt_VAL = 0.0;                  //Hodnota naměřeného napětí 
float           Temp_M = 0.0;                    //Hodnota naměřené teploty motoru
float           Temp_V = 0.0;                    //Hodnota naměřené venkovní teploty
unsigned long   Stary_Cas = 0;                   //Hodnota času (v ms) pro kontrolu délky měření otáček
float           Cas_Sekundy = 0.000;             //Hodnota času prevedeného z milisekund na sekundy


//Vytvoření ikon a deklarace polí
byte PPznak[8]                                   //Deklarace znaku PP
{
 0b11100, 0b10100, 0b11100, 0b10000, 0b10111, 0b00101, 0b00111, 0b00100   //znak PP
}; 


//***********************************************************
//****** Část programu - vykonávající po resetu *************
//***********************************************************
void setup() 
{
  pinMode(DS18B20m_Pin, INPUT);           //Inicializace digitálního vstupu - teploměr DS1820
  pinMode(DS18B20v_Pin, INPUT);           //Inicializace digitálního vstupu - teploměr DS1820
  pinMode(RPM_Pulzy, INPUT);              //Inicializace digitálního vstupu - měření frekvence-otáček-pulsu 
  pinMode(Volba1, INPUT_PULLUP);          //Inicializace digitálního vstupu s vnitřním rezistorem
  pinMode(Volba2, INPUT_PULLUP);          //Inicializace digitalniho vstupu s vnitřním rezistorem
  pinMode(Volba3, INPUT_PULLUP);          //Inicializace digitálního vstupu s vnitřním rezistorem
  pinMode(Volba4, INPUT_PULLUP);          //Inicializace digitalniho vstupu s vnitřním rezistorem

  attachInterrupt(pinINT0, prictiPulz, FALLING);   //Vnější přerušení aktivace od sestupné hrany, volá podprogram "prictiPulz"
  
  lcd1.begin(8, 2);                       //Definice LCD displeje č.1 8 znaků, 2 řádky
  lcd2.begin(8, 2);                       //Definice LCD displeje č.2 8 znaků, 2 řádky
  lcd1.createChar(1,PPznak);              //Vytvoření znaků do paměti displeje č.1 

  lcd1.clear();                           //Vymazaní displeje č.1
  lcd2.clear();                           //Vymazaní displeje č.2
  lcd1.setCursor(7, 1);                   //Nastavení kurzoru na 8.pozici druhého řádku
  lcd1.write(1);                          //Zobrazení vytvořeného znaku PP
  lcd1.setCursor(0, 0);                   //Nastavení kurzoru na 1.pozici prvního řádku   
  lcd1.print("R");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("-");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("T");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("-");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("V");                        //Zobrazení nápisu na LCD č.1
  delay(200);                             //Zpoždovací smyčka 0,2s
  lcd1.setCursor(0, 1);                   //Nastavení kurzoru na 1.pozici druhého řádku
  lcd1.print("M");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("E");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("T");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("E");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd1.print("R");                        //Zobrazení nápisu na LCD č.1
  delay(100);                             //Zpoždovací smyčka 0,1s
  lcd2.setCursor(0, 0);                   //Nastavení kurzoru na 1.pozici prvního řádku LCD č.2 
  Volba_pulz_otacka();                    //Volání podprogramu pro nastavení počtu pulzů na otáčku            
  lcd2.print("v2,0 p:");                  //Zobrazení nápisu na LCD č.2 
  lcd2.print(Pulz_Otacka);                //Zobrazení nastaveného počtu impulzů na otáčku 
  lcd2.setCursor(0, 1);                   //Nastavení kurzoru na 1.pozici druhého řádku LCD č.2
  lcd2.print("p78c2017");                 //Zobrazení napisu na LCD č.2
  delay(1000);                            //Čekací smyčka 1s
  lcd1.clear();                           //Vymazani displeje c.1
  lcd2.clear();                           //Vymazani displeje c.2

}  //konec části programu po resetu


//***********************************************************
//****** Část programu - obsluha přerušení  *****************
//*********************************************************** 
void prictiPulz() 
{    
  Pocet_Pulzu++;          //inkrementace čítače pulzů
  
} //konec části programu po přerušení


//***********************************************************
//****** Podprogram - nastavení pulzu na otáčku         *****
//*********************************************************** 
void Volba_pulz_otacka() 
{    
  //Nastavení jeden pulz na otáčku - defaultní hodnota v deklaraci proměných
  //Volba3 a Volba4 = HIGH  .... tj. 1 pulz na otáčku
  //Nastavení dva pulzy na otáčku
  if  (digitalRead(Volba3)== LOW && digitalRead(Volba4)== HIGH)
    {
      Pulz_Otacka = 2;                 //Hodnota udávající počet pulzů na jednu otáčku
    }
  //Nastavení tři pulzy na otáčku
  if  (digitalRead(Volba3)== HIGH && digitalRead(Volba4)== LOW)
    {
      Pulz_Otacka = 3;                 //Hodnota udávající počet pulzů na jednu otáčku
    }
  //Nastavení čtyři pulzy na otáčku
  if  (digitalRead(Volba3)== LOW && digitalRead(Volba4)== LOW)
    {
      Pulz_Otacka = 4;                 //Hodnota udávající počet pulzů na jednu otáčku
    }
 
} //konec části podprogramu volby pulzu na otáčku


//***********************************************************
//****** Cast programu - nekonecna smycka              ******
//*********************************************************** 
void loop() 
{
    Mereni_Otacek();                  //Volani podprogramu pro mereni otacek motoru
    Zobrazeni_Otacek();               //Volani podprogramu pro zobrazeni otacek motoru na LCD displeji 
    Mereni_Napeti();                  //Volani podprogramu pro mereni napeti z AD prevodniku
    Zobrazeni_Napeti();               //Volani podprogramu pro zobrazeni napeti na LCD displeji
    Mereni_Teploty();                 //Volani podprogramu pro mereni teploty z DS1820
    Zobrazeni_Teploty();              //Volani podprogramu pro zobrazeni teploty z DS1820 na displeji

    //Zobrazení znaků ":" a " " z důvodu zpomalení aktualizace hodnot a informace a funkčnosti
    lcd2.setCursor(1, 0);             //Nastavení kurzoru na 2.pozici prvního řádku LCD č.2
    lcd2.print(":");                  //Zobrazení napisu na LCD č.2
    delay(300);                       //Čekací smyčka 0,3s
    lcd2.setCursor(1, 0);             //Nastavení kurzoru na 2.pozici prvního řádku LCD č.2
    lcd2.print(" ");                  //Zobrazení napisu na LCD č.2
    delay(500);                       //Čekací smyčka 0,5s
         
}  //konec části programu nekonečné smyčky


//***********************************************************
//*** Podprogram pro měření otáček motoru             *******
//***********************************************************
void Mereni_Otacek(void)
{
  if ((millis() - Stary_Cas) >= 1000)               //Je-li rozdíl casu 1 sekunda a vice proved mereni a prepocet 
      {
            detachInterrupt(pinINT0);               //Vypnutí detekce přerušení po dobu výpočtu a tisku výsledku
            Cas_Sekundy = millis() - Stary_Cas;     //Odečet aktuálního naměřeného času v milisekundách
            Cas_Sekundy = Cas_Sekundy / 1000;       //Převod aktuálního času z milisekund na sekundy 
            RPM_VAL = Pocet_Pulzu / (Cas_Sekundy * Pulz_Otacka);      //Výpočet frekvence podle poctu pulzu za dany cas           
            RPM_VAL = RPM_VAL * 60;                 //Výpočet otacek podle vypočtené frekvence

            //Výpočet otáček se zaokrouhlením po 50ot/min
            if  (digitalRead(Volba2)== LOW)         //Je-li vstup "Volba2" aktivní, skoč na zaokrouhlení měřených otáček
               {
                 Zaokrouhleni_Otacek();             //Volání podprogramu pro zaokrouhleni merenych otacek
                }
            
            Pocet_Pulzu = 0;                        //Vynulování čítače impulzů pro měření frekvence
            Stary_Cas = millis();                   //Zapamatování času pro následující výpočet
            attachInterrupt(pinINT0, prictiPulz, FALLING);   //Vnější přerušení aktivace od sestupné hrany, volá podprogram "prictiPulz"
      }

}  //konec části podprogramu pro měření otáček motoru

//***********************************************************
//****** Podprogram - zaokrouhlení otáček po 50ot/min   *****
//*********************************************************** 
void Zaokrouhleni_Otacek() 
{    
    RPM_VAL_INT = RPM_VAL;                     //Přesun naměřených otáček k dalšímu výpočtu včetně přetypování z FLOAT na INT  
    RPM_Modulo = RPM_VAL_INT % 100;            //Výpočet zbytku (typu modulo) po dělení otáček číslem 100 
    if  (RPM_Modulo <= 25)                     //Je-li výsledek modulo <= 25, nastav konstantu 0 pro přičtení
       {
          RPM_Pricti = 0;                      //Nastavení hodnoty nula pro přičtení 
        }
    if  (RPM_Modulo > 26 && RPM_Modulo <= 75)  //Je-li výsledek modulo mezi 26 a 75, nastav konstantu 50 pro přičtení
       {
          RPM_Pricti = 50;                     //Nastavení hodnoty 50 pro přičtení
        }
    if  (RPM_Modulo > 76)                      //Je-li výsledek modulo > 76, nastav konstantu 100 pro přičtení
       {
          RPM_Pricti = 100;                    //Nastavení hodnoty 100 pro přičtení
        }
    RPM_VAL_INT = RPM_VAL_INT / 100;           //Vydělení měřených otáček číslem 100 = ořez aktuálních jednotek a desítek
    RPM_VAL_INT = RPM_VAL_INT * 100;           //Vynásobení měřených otáček číslem 100, místo jednotek a desítek jsou nuly
    RPM_VAL = RPM_VAL_INT + RPM_Pricti;        //Přičtení konstanty 0,50,100 k otáčkám a přetypování zpět z int na float
    
} //konec části podprogramu pro zaokrouhleni otacek po 50ot/min

//***********************************************************
//*** Podprogram pro zobrazení otáček motoru          *******
//***********************************************************
void Zobrazeni_Otacek(void)
{
    lcd1.setCursor(0, 0);                     //Nastavení kurzoru na 1.pozici prvního řádku LCD č.1
    lcd1.print("     ");                      //Zobrazení napisu na LCD č.1 - vymazání předešlé hodnoty 
    lcd1.setCursor(4, 0);                     //Nastavení kurzoru na 5.pozici prvního řádku LCD č.1  
    if  (RPM_VAL > 9)                         //Jsou-li otáčky větší než 9, nastav 4.pozici zobrazení
      {
        lcd1.setCursor(3, 0);                 //Nastavení kurzoru na 4.pozici prvního řádku LCD č.1  
      }
    if  (RPM_VAL > 99)                        //Jsou-li otáčky větší než 99, nastav 3.pozici zobrazení
      {
        lcd1.setCursor(2, 0);                 //Nastavení kurzoru na 3.pozici prvního řádku LCD č.1  
      }
    if  (RPM_VAL > 999)                       //Jsou-li otáčky větší než 999, nastav 2.pozici zobrazení
      {
        lcd1.setCursor(1, 0);                 //Nastavení kurzoru na 2.pozici prvního řádku LCD č.1  
      } 
    if  (RPM_VAL > 9999)                      //Jsou-li otáčky větší než 9999, nastav 1.pozici zobrazení
      {
        lcd1.setCursor(0, 0);                 //Nastavení kurzoru na 1.pozici prvního řádku LCD č.1  
      }
    if  (RPM_VAL > 50000)
      {
        lcd1.print("^^^^^");                  //Zobrazení napisu na LCD č.1
      }
      else
      {
        lcd1.print(RPM_VAL,0);                //Zobrazení otáček motoru na LCD č.1 
      }
    lcd1.setCursor(6, 0);                     //Nastavení kurzoru na 7.pozici prvního řádku LCD č.1
    lcd1.print("ot");                         //Zobrazení napisu na LCD č.1
}  //konec části podprogramu pro zobrazení otáček motoru


//***********************************************************
//*** Podprogram pro měření napětí                    *******
//***********************************************************
void Mereni_Napeti(void)
{
    Volt_VAL = (analogRead(Voltmetr));        //Cteni hodnoty napeti z AD prevodniku
    Volt_VAL = (Volt_VAL * 0.0488);           //Prevod na hodnoty na napeti, rozsah 0 až 50V
     
}  //konec části podprogramu pro měření napětí


//***********************************************************
//*** Podprogram pro zobrazení napětí                 *******
//***********************************************************
void Zobrazeni_Napeti(void)
{
    lcd2.setCursor(0, 0);                     //Nastavení kurzoru na 1.pozici prvního řádku LCD č.2
    lcd2.print("B");                          //Zobrazení napisu na LCD č.2
    lcd2.setCursor(3, 0);                     //Nastavení kurzoru na 4.pozici prvního řádku LCD č.2
    lcd2.print(Volt_VAL,1);                   //Zobrazení teploty motoru na jedno desetinné místo na LCD č.2
    lcd2.print("V ");                         //Zobrazení napisu na LCD č.2
    
}  //konec části podprogramu pro zobrazení napětí


//***********************************************************
//*** Podprogram pro měření teploty                   *******
//***********************************************************
void Mereni_Teploty(void)
{
    Temp_M = getTemp_M();            //volani podprogramu pro čtení hodnot z teploměru motoru
    Temp_V = getTemp_V();            //volani podprogramu pro čtení hodnot z venkovního teploměru
     
}  //konec části podprogramu pro měření teploty


//***********************************************************
//*** Podprogram pro zobrazení teploty                *******
//***********************************************************
void Zobrazeni_Teploty(void)
{
  //Část pro zobrazení teploty motoru na displeji č.1
  lcd1.setCursor(0, 1);                   //Nastavení kurzoru na 1.pozici druhého řádku LCD č.1
  lcd1.print("M");                        //Zobrazení napisu na LCD č.1
   if  (Temp_M >= 0 && Temp_M < 100)      //Je-li měřená teplota >0degC a <100degC, posun zobrazení o jednu vpravo
      {
        lcd1.print(" ");                  //Zobrazení napisu na LCD č.1
      }
    if  (Temp_M > -90)                    //Je-li měřená teplota > -90degC, zobraz teplotu, jinak zobraz nepřipojený teploměr
      {
        lcd1.print(Temp_M,1);             //Zobrazení teploty motoru na jedno desetinné místo na LCD č.1 
      }
      else
      {
        lcd1.setCursor(2, 1);             //Nastavení kurzoru na 3.pozici druhého řádku LCD č.1
        lcd1.print("__._");               //Zobrazení napisu něpřipojeného teploměru na LCD č.1
      }
  lcd1.setCursor(6, 1);                   //Nastavení kurzoru na 7.pozici druhého řádku LCD č.1  
  lcd1.print(char(223));                  //Zobrazení znaku stupeň celsia na LCD č.1 
  lcd1.print("C");                        //Zobrazení napisu na LCD č.1

  //Část pro zobrazení teploty motoru na displeji č.2
  lcd2.setCursor(0, 1);                   //Nastavení kurzoru na 1.pozici druhého řádku LCD č.2
  lcd2.print("V");                        //Zobrazení napisu na LCD č.2
   if  (Temp_V >= 0 && Temp_V < 100)      //Je-li měřená teplota >0degC a <100degC, posun zobrazení o jednu vpravo
      {
        lcd2.print(" ");                  //Zobrazení napisu na LCD č.2
      }
    if  (Temp_V > -90)                    //Je-li měřená teplota > -90degC, zobraz teplotu, jinak zobraz nepřipojený teploměr
      {
        lcd2.print(Temp_V,1);             //Zobrazení teploty motoru na jedno desetinné místo na LCD č.2 
      }
      else
      {
        lcd2.setCursor(2, 1);             //Nastavení kurzoru na 3.pozici druhého řádku LCD č.2
        lcd2.print("__._");               //Zobrazení napisu něpřipojeného teploměru na LCD č.2
      }
  lcd2.setCursor(6, 1);                   //Nastavení kurzoru na 7.pozici druhého řádku LCD č.2  
  lcd2.print(char(223));                  //Zobrazení znaku stupeň celsia na LCD č.2
  lcd2.print("C");                        //Zobrazení napisu na LCD č.2
  
}  //konec části podprogramu pro zobrazení teploty


//***********************************************************
//*** Podprogram pro čtení teploty z DS1820           *******
//***********************************************************
//Část pro čtení hodnot z teploměru motoru
float getTemp_M()
{
  byte data[12];
  byte addr[8];
  if ( !ds1.search(addr)) 
      {
        ds1.reset_search();
        return -1000;
      }
  if ( OneWire::crc8( addr, 7) != addr[7]) 
      {
        return -1000;
      }
  if ( addr[0] != 0x10 && addr[0] != 0x28) 
      {
        return -1000;
      }
  ds1.reset();
  ds1.select(addr);
  ds1.write(0x44,1);
  byte present = ds1.reset();
  ds1.select(addr);
  ds1.write(0xBE);
  for (int i = 0; i < 9; i++) // potřebujeme 9 bytů
    { 
      data[i] = ds1.read();
     }
  ds1.reset_search();
 
  byte MSB = data[1];
  byte LSB = data[0];
  int16_t raw = ((MSB << 8) | LSB);    //Spojeni dvou bytu MSB a LSB do jednoho WORDU

  //Vyhodnoceni jaky typ teplomeru je pripojen - prvni byte pameti ROM 10h = DS18S20 a starsi, 28h = DS18B20
  //DS18S20 a starsi pouzivaji pouze 9ti bitove cislo nesouci hodnotu teploty
  if  (addr[0] == 0x10)          //Je-li prvni adresa ROM 10h - DS18S20 a starsi
          {
            raw = raw << 3;       // 9 bit resolution default
            raw = (raw & 0xFFF0) + 12 - data[6];
          } 

  float TemperatureSum = (float) raw / 16;
  return TemperatureSum;
} //konec podprogramu pro cteni hodnot z teplomeru DS18B20

//Část pro čtení hodnot z venkovního teploměru
  float getTemp_V()
{
  byte data[12];
  byte addr[8];
  if ( !ds2.search(addr)) 
      {
        ds2.reset_search();
        return -1000;
      }
  if ( OneWire::crc8( addr, 7) != addr[7]) 
      {
        return -1000;
      }
  if ( addr[0] != 0x10 && addr[0] != 0x28)
      {
        return -1000;
      }
  ds2.reset();
  ds2.select(addr);
  ds2.write(0x44,1);
  byte present = ds2.reset();
  ds2.select(addr);
  ds2.write(0xBE);
  for (int i = 0; i < 9; i++)   // potřebujeme 9 bytů
    { 
      data[i] = ds2.read();
     }
  ds2.reset_search();
 
  byte MSB = data[1];
  byte LSB = data[0];
  int16_t raw = ((MSB << 8) | LSB);    //Spojeni dvou bytu MSB a LSB do jednoho WORDU

  //Vyhodnoceni jaky typ teplomeru je pripojen - prvni byte pameti ROM 10h = DS18S20 a starsi, 28h = DS18B20
  //DS18S20 a starsi pouzivaji pouze 9ti bitove cislo nesouci hodnotu teploty
  if  (addr[0] == 0x10)          //Je-li prvni adresa ROM 10h - DS18S20 a starsi
          {
            raw = raw << 3;       // 9 bit resolution default
            raw = (raw & 0xFFF0) + 12 - data[6];
          } 

  float TemperatureSum = (float) raw / 16;
  return TemperatureSum;
} //konec podprogramu pro cteni hodnot z teplomeru DS18B20
